@@ -1,6 +1,6 @@
 /* Core dump and executable file functions below target vector, for GDB.
 
-   Copyright (C) 1986-2013 Free Software Foundation, Inc.
+   Copyright (C) 1986-2014 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -28,6 +28,7 @@
 #endif
 #include "frame.h"		/* required by inferior.h */
 #include "inferior.h"
+#include "infrun.h"
 #include "symtab.h"
 #include "command.h"
 #include "bfd.h"
@@ -85,7 +86,7 @@ static int gdb_check_format (bfd *);
 
 static void core_open (char *, int);
 
-static void core_close (void);
+static void core_close (struct target_ops *self);
 
 static void core_close_cleanup (void *ignore);
 
@@ -192,7 +193,7 @@ gdb_check_format (bfd *abfd)
    stack spaces as empty.  */
 
 static void
-core_close (void)
+core_close (struct target_ops *self)
 {
   if (core_bfd)
     {
@@ -223,7 +224,7 @@ core_close (void)
 static void
 core_close_cleanup (void *ignore)
 {
-  core_close ();
+  core_close (NULL);
 }
 
 /* Look for sections whose names start with `.reg/' so that we can
@@ -348,7 +349,7 @@ core_open (char *filename, int from_tty)
 
   validate_files ();
 
-  core_data = XZALLOC (struct target_section_table);
+  core_data = XCNEW (struct target_section_table);
 
   /* Find the data section */
   if (build_section_table (core_bfd,
@@ -649,7 +650,7 @@ add_to_spuid_list (bfd *abfd, asection *asect, void *list_p)
    the to_xfer_partial interface.  */
 
 static LONGEST
-get_core_siginfo (bfd *abfd, gdb_byte *readbuf, ULONGEST offset, LONGEST len)
+get_core_siginfo (bfd *abfd, gdb_byte *readbuf, ULONGEST offset, ULONGEST len)
 {
   asection *section;
   char *section_name;
@@ -672,17 +673,17 @@ get_core_siginfo (bfd *abfd, gdb_byte *readbuf, ULONGEST offset, LONGEST len)
   return len;
 }
 
-static LONGEST
+static enum target_xfer_status
 core_xfer_partial (struct target_ops *ops, enum target_object object,
 		   const char *annex, gdb_byte *readbuf,
 		   const gdb_byte *writebuf, ULONGEST offset,
-		   LONGEST len)
+		   ULONGEST len, ULONGEST *xfered_len)
 {
   switch (object)
     {
     case TARGET_OBJECT_MEMORY:
       return section_table_xfer_memory_partial (readbuf, writebuf,
-						offset, len,
+						offset, len, xfered_len,
 						core_data->sections,
 						core_data->sections_end,
 						NULL);
@@ -698,25 +699,28 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 
 	  section = bfd_get_section_by_name (core_bfd, ".auxv");
 	  if (section == NULL)
-	    return -1;
+	    return TARGET_XFER_E_IO;
 
 	  size = bfd_section_size (core_bfd, section);
 	  if (offset >= size)
-	    return 0;
+	    return TARGET_XFER_EOF;
 	  size -= offset;
 	  if (size > len)
 	    size = len;
-	  if (size > 0
-	      && !bfd_get_section_contents (core_bfd, section, readbuf,
-					    (file_ptr) offset, size))
+
+	  if (size == 0)
+	    return TARGET_XFER_EOF;
+	  if (!bfd_get_section_contents (core_bfd, section, readbuf,
+					 (file_ptr) offset, size))
 	    {
 	      warning (_("Couldn't read NT_AUXV note in core file."));
-	      return -1;
+	      return TARGET_XFER_E_IO;
 	    }
 
-	  return size;
+	  *xfered_len = (ULONGEST) size;
+	  return TARGET_XFER_OK;
 	}
-      return -1;
+      return TARGET_XFER_E_IO;
 
     case TARGET_OBJECT_WCOOKIE:
       if (readbuf)
@@ -730,7 +734,7 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 
 	  section = bfd_get_section_by_name (core_bfd, ".wcookie");
 	  if (section == NULL)
-	    return -1;
+	    return TARGET_XFER_E_IO;
 
 	  size = bfd_section_size (core_bfd, section);
 	  if (offset >= size)
@@ -738,27 +742,39 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 	  size -= offset;
 	  if (size > len)
 	    size = len;
-	  if (size > 0
-	      && !bfd_get_section_contents (core_bfd, section, readbuf,
-					    (file_ptr) offset, size))
+
+	  if (size == 0)
+	    return TARGET_XFER_EOF;
+	  if (!bfd_get_section_contents (core_bfd, section, readbuf,
+					 (file_ptr) offset, size))
 	    {
 	      warning (_("Couldn't read StackGhost cookie in core file."));
-	      return -1;
+	      return TARGET_XFER_E_IO;
 	    }
 
-	  return size;
+	  *xfered_len = (ULONGEST) size;
+	  return TARGET_XFER_OK;
+
 	}
-      return -1;
+      return TARGET_XFER_E_IO;
 
     case TARGET_OBJECT_LIBRARIES:
       if (core_gdbarch
 	  && gdbarch_core_xfer_shared_libraries_p (core_gdbarch))
 	{
 	  if (writebuf)
-	    return -1;
-	  return
-	    gdbarch_core_xfer_shared_libraries (core_gdbarch,
-						readbuf, offset, len);
+	    return TARGET_XFER_E_IO;
+	  else
+	    {
+	      *xfered_len = gdbarch_core_xfer_shared_libraries (core_gdbarch,
+								readbuf,
+								offset, len);
+
+	      if (*xfered_len == 0)
+		return TARGET_XFER_EOF;
+	      else
+		return TARGET_XFER_OK;
+	    }
 	}
       /* FALL THROUGH */
 
@@ -767,10 +783,19 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 	  && gdbarch_core_xfer_shared_libraries_aix_p (core_gdbarch))
 	{
 	  if (writebuf)
-	    return -1;
-	  return
-	    gdbarch_core_xfer_shared_libraries_aix (core_gdbarch,
-						    readbuf, offset, len);
+	    return TARGET_XFER_E_IO;
+	  else
+	    {
+	      *xfered_len
+		= gdbarch_core_xfer_shared_libraries_aix (core_gdbarch,
+							  readbuf, offset,
+							  len);
+
+	      if (*xfered_len == 0)
+		return TARGET_XFER_EOF;
+	      else
+		return TARGET_XFER_OK;
+	    }
 	}
       /* FALL THROUGH */
 
@@ -789,23 +814,26 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 
 	  section = bfd_get_section_by_name (core_bfd, sectionstr);
 	  if (section == NULL)
-	    return -1;
+	    return TARGET_XFER_E_IO;
 
 	  size = bfd_section_size (core_bfd, section);
 	  if (offset >= size)
-	    return 0;
+	    return TARGET_XFER_EOF;
 	  size -= offset;
 	  if (size > len)
 	    size = len;
-	  if (size > 0
-	      && !bfd_get_section_contents (core_bfd, section, readbuf,
-					    (file_ptr) offset, size))
+
+	  if (size == 0)
+	    return TARGET_XFER_EOF;
+	  if (!bfd_get_section_contents (core_bfd, section, readbuf,
+					 (file_ptr) offset, size))
 	    {
 	      warning (_("Couldn't read SPU section in core file."));
-	      return -1;
+	      return TARGET_XFER_E_IO;
 	    }
 
-	  return size;
+	  *xfered_len = (ULONGEST) size;
+	  return TARGET_XFER_OK;
 	}
       else if (readbuf)
 	{
@@ -818,21 +846,37 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
 	  list.pos = 0;
 	  list.written = 0;
 	  bfd_map_over_sections (core_bfd, add_to_spuid_list, &list);
-	  return list.written;
+
+	  if (list.written == 0)
+	    return TARGET_XFER_EOF;
+	  else
+	    {
+	      *xfered_len = (ULONGEST) list.written;
+	      return TARGET_XFER_OK;
+	    }
 	}
-      return -1;
+      return TARGET_XFER_E_IO;
 
     case TARGET_OBJECT_SIGNAL_INFO:
       if (readbuf)
-	return get_core_siginfo (core_bfd, readbuf, offset, len);
-      return -1;
+	{
+	  LONGEST l = get_core_siginfo (core_bfd, readbuf, offset, len);
+
+	  if (l > 0)
+	    {
+	      *xfered_len = len;
+	      return TARGET_XFER_OK;
+	    }
+	}
+      return TARGET_XFER_E_IO;
 
     default:
       if (ops->beneath != NULL)
 	return ops->beneath->to_xfer_partial (ops->beneath, object,
 					      annex, readbuf,
-					      writebuf, offset, len);
-      return -1;
+					      writebuf, offset, len,
+					      xfered_len);
+      return TARGET_XFER_E_IO;
     }
 }
 
@@ -842,7 +886,8 @@ core_xfer_partial (struct target_ops *ops, enum target_object object,
    breakpoint_init_inferior).  */
 
 static int
-ignore (struct gdbarch *gdbarch, struct bp_target_info *bp_tgt)
+ignore (struct target_ops *ops, struct gdbarch *gdbarch,
+	struct bp_target_info *bp_tgt)
 {
   return 0;
 }
@@ -869,10 +914,16 @@ static const struct target_desc *
 core_read_description (struct target_ops *target)
 {
   if (core_gdbarch && gdbarch_core_read_description_p (core_gdbarch))
-    return gdbarch_core_read_description (core_gdbarch, 
-					  target, core_bfd);
+    {
+      const struct target_desc *result;
 
-  return NULL;
+      result = gdbarch_core_read_description (core_gdbarch, 
+					      target, core_bfd);
+      if (result != NULL)
+	return result;
+    }
+
+  return target->beneath->to_read_description (target->beneath);
 }
 
 static char *
@@ -949,14 +1000,12 @@ init_core_ops (void)
     "Use a core file as a target.  Specify the filename of the core file.";
   core_ops.to_open = core_open;
   core_ops.to_close = core_close;
-  core_ops.to_attach = find_default_attach;
   core_ops.to_detach = core_detach;
   core_ops.to_fetch_registers = get_core_registers;
   core_ops.to_xfer_partial = core_xfer_partial;
   core_ops.to_files_info = core_files_info;
   core_ops.to_insert_breakpoint = ignore;
   core_ops.to_remove_breakpoint = ignore;
-  core_ops.to_create_inferior = find_default_create_inferior;
   core_ops.to_thread_alive = core_thread_alive;
   core_ops.to_read_description = core_read_description;
   core_ops.to_pid_to_str = core_pid_to_str;
